/*
All interaction with the server is done via:

/_/command;command;command;command

There can be any reasonable number of commands, separated by semicolon. They will be executed in order, and a response may be given, depending on the commands.

The format of the response is a list of lines (separated by \n), and each line
has tokens separated by \t. 

Valid commands:

* numeric values 0-65535 
REAPER command IDs. (todo: support for registered names, too). These typically do not have any response.

* TRANSPORT
returns a line including (note that the spaces are here for readability, there
is actually only tabs between fields):
  TRANSPORT\t playstate\t position_seconds\t isRepeatOn

playstate is 0 for stopped, 1 for playing, 2 for paused, 5 for recording, and 6 for record paused.

isRepeatOn will be nonzero if repeat is enabled

* NTRACK

Requests number of tracks. Returned will be:

  NTRACK\tvalue

value is 0 (no tracks) or greater. 

* TRACK  or   TRACK/index   or TRACK/start-end

Requests information about all tracks, a single track, or a range of tracks.
Note that the indices used are 0 for master, 1 for first user track, etc.

Returns zero or more lines:

  TRACK\t tracknumber\t trackname\t trackflags\t volume\t pan\t last_meter_peak\t last_meter_pos

    tracknumber is 0 for master, 1 for first track, etc.
    trackname is the name of the track, or MASTER
    trackflags includes various bits including mute,solo, etc state, see trackFlagsToString() below
    volume is track volume, where 0 is -inf, 1 is +0dB, etc. see mkvolstr for dB conversions
    pan is -1..1, where -1 is full left, 0 is centered, 1 is full right
    last_meter_peak and last_meter_pos are integers that are dB*10, so -100 would be -10dB.
  

* GET/TRACK/index/xxxxxx

Requests information for track "index", via GetSetMediaTrackInfo(). See the REAPER API documentation for which
strings are acceptable, but for example you can query the track record input index for the first track via:

  GET/TRACK/1/I_RECINPUT

The returned string will be:

GET/TRACK/1/I_RECINPUT\t <index>

(if an error occurs you may get nothing back at all, or you might get the GET string back but with no parameter)


* SET/TRACK/index/xxxxx/value

Similar to GET/TRACK/index/xxxxx, but sets the value of this item. You probably will want to follow this with a SET/UNDO command, below. This will not give any response.


* SET/TRACK/index/VOL/value

Special case of SET/TRACK/index/xxxx, sets volume for a track via control surface API (meaning it respects automation modes etc). If value starts with + or -, then adjustment is relative, and in 100ths of fader range, otherwise adjustment is absolute (1=0dB, etc). If value ends in "g", then ganging is ignored. Does not need SET/UNDO.

* SET/TRACK/index/PAN/value

Special case of SET/TRACK/index/xxxx, sets pan for a track via control surface API. If value starts with + or -, adjustment is relative. Range is always -1..1. If value ends in "g", then ganging is ignored. Does not need SET/UNDO.

* SET/TRACK/index/MUTE/value

Special case of SET/TRACK/index/xxxx, sets mute for track. if value is <0, mute is toggled. Does not need SET/UNDO.

* SET/TRACK/index/SOLO/value

Special case of SET/TRACK/index/xxxx, sets solo for track. if value is <0, solo is toggled. Does not need SET/UNDO.

* SET/TRACK/index/FX/value

Special case of SET/TRACK/index/xxxx, sets fx enabled for track. if value is <0, fx enabled is toggled. Does not need SET/UNDO.

* SET/TRACK/index/RECARM/value

Special case of SET/TRACK/index/xxxx, sets record arm enabled for track. if value is <0, record arm enabled is toggled. Does not need SET/UNDO.

* SET/TRACK/index/SEL/value

Special case of SET/TRACK/index/xxxx, sets selection for track. if value is <0, selection is toggled. Does not need SET/UNDO.



* SET/UNDO/message

Adds an undo point, useful if you have modified anything that needs it.


* SET/REPEAT/val

If val is -1, toggles repeat, 0 disables repeat, 1 enables repeat.

* GET/REPEAT

Returns GET/REPEAT\tval  where val is 0 or 1.

*/

function initxmlhttp()
{
var xmlhttp=false;
/*@cc_on @*/
/*@if (@_jscript_version >= 5)
// JScript gives us Conditional compilation, we can cope with old IE versions.
// and security blocked creation of the objects.
 try {
  xmlhttp = new ActiveXObject("Msxml2.XMLHTTP");
 } catch (e) {
  try {
   xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
  } catch (E) {
   xmlhttp = false;
  }
 }
@end @*/
  if (!xmlhttp && typeof XMLHttpRequest!='undefined')
  {
        try {
                xmlhttp = new XMLHttpRequest();
        } catch (e) {
                xmlhttp=false;
        }
  }
  if (!xmlhttp && window.createRequest)
  {
        try {
                xmlhttp = window.createRequest();
        } catch (e) {
                xmlhttp=false;
        }
  }
  return xmlhttp;
}


var g_wwr_timer_freq=100;
var g_wwr_req_list = "", g_wwr_req_list2 = "";
var g_wwr_req_recur = new Array();
var g_wwr_req = null;
var g_wwr_timer = null;
var g_wwr_timer2 = null;
function wwr_run_update()
{
  g_wwr_timer = null;
  if (!g_wwr_req) g_wwr_req = initxmlhttp();
  if (!g_wwr_req) 
  {
    alert("no xml http support");
    return;
  }

  // populate any periodic requests
  var x;
  var d = (new Date).getTime();
  for (x=0;x<g_wwr_req_recur.length;x++)
  {
    if (g_wwr_req_recur[x][2] < d)
    {
      g_wwr_req_recur[x][2] = d + g_wwr_req_recur[x][1];
      g_wwr_req_list2 += g_wwr_req_recur[x][0] + ";";
    }
  }
  g_wwr_req_list += g_wwr_req_list2;
  g_wwr_req_list2 = "";

  if (g_wwr_req_list != "")
  {
    g_wwr_req.open("GET","/_/" + g_wwr_req_list,true);
    g_wwr_req.onreadystatechange=function() 
    {
      if (g_wwr_req.readyState==4) 
      {
        if (g_wwr_timer2) { clearTimeout(g_wwr_timer2); g_wwr_timer2=null; }
        wwr_onreply(g_wwr_req.responseText);
        wwr_run_update();
      }
    };

    if (g_wwr_timer2) clearTimeout(g_wwr_timer2); 
    g_wwr_timer2 = window.setTimeout(function()
    {
      g_wwr_timer2=null;
      if (g_wwr_req.readyState!=0 && g_wwr_req.readyState!=4)
      {
         if (g_wwr_timer) { clearTimeout(g_wwr_timer); g_wwr_timer=null; }
         g_wwr_req.abort();
         wwr_run_update();
      }
    },3000);
    
    g_wwr_req_list = "";
    g_wwr_req.send(null);
  }
  else
  {
    g_wwr_timer = window.setTimeout(wwr_run_update,g_wwr_timer_freq);
  }
}


function wwr_start()
{
   wwr_run_update();
}
function wwr_req(name)
{
  g_wwr_req_list += name + ";";
  
  /*
  // force immediate
  if (g_wwr_timer)
  {
    window.clearTimeout(g_wwr_timer);
    g_wwr_timer=null;
  }
  wwr_run_update();
  */
}

function wwr_req_recur(name, interval)
{
  g_wwr_req_recur.push([name,interval,0]);
}

function trackFlagsToString(flags)
{
  var ret="";
  if (flags&1) ret += "folder, ";
  if (flags&2) ret += "selected, ";
  if (flags&4) ret += "has fx, ";
  if (flags&8) ret += "muted, ";
  if (flags&16) 
  {
    ret += "soloed";
    if (flags&32) ret += " (SIP)";
    ret += ", ";
  }
  if (flags&64) ret += "record armed, "
  var len = ret.length;
  if (ret.substring(len-2) == ", ") ret=ret.substring(0,len-2);
  return ret;
}

function mkvolstr(vol)
{
  var v = parseFloat(vol);
  if (v < 0.00000002980232) return "-inf dB";
  v = Math.log(v)*8.68588963806;
  return v.toFixed(2) + " dB";
}

function mkpanstr(pan)
{
  if (Math.abs(pan) < 0.001) return "center";
  if (pan > 0) return (pan*100).toFixed(0) + "%R";
  return (pan*-100).toFixed(0) + "%L";
}
